@@ -171,7 +171,7 @@ assert_expansion_function() {
   local ws=${PWD}
   local function="$1" displacement="$2" type="$3" expected="$4" current="$5"
   disable_errexit
-  local actual_result=$(eval "_bazel__${function} \"${ws}\" \"${displacement}\" \"${current}\" \"${type}\"")
+  local actual_result=$(eval "_bazel__${function} \"${ws}\" \"${displacement}\" \"${current}\" \"${type}\"" | sort)
   enable_errexit
   assert_equals "$(echo -ne "${expected}")" "${actual_result}"
 }
@@ -196,7 +196,7 @@ test_expand_rules_in_package() {
 
     # label should match test and non-test rules
     assert_expansion_function "expand_rules_in_package" "" label \
-    'token_bucket_test \ntoken_bucket_binary ' \
+    'token_bucket_binary \ntoken_bucket_test ' \
     'video/streamer2:token_bucket_'
     assert_expansion_function "expand_rules_in_package" "" label \
     'stuff ' 'video/streamer2/stuff:s'
@@ -234,7 +234,7 @@ test_expand_rules_in_package() {
     # with BAZEL_COMPLETION_ALLOW_TESTS_FOR_RUN set.
     BAZEL_COMPLETION_ALLOW_TESTS_FOR_RUN=true \
     assert_expansion_function "expand_rules_in_package" "" label-bin \
-    'token_bucket_test \ntoken_bucket_binary ' 'video/streamer2:to'
+    'token_bucket_binary \ntoken_bucket_test ' 'video/streamer2:to'
 
     # Test the label-bin expands for test rules, with
     # BAZEL_COMPLETION_ALLOW_TESTS_FOR_RUN set.
@@ -303,7 +303,7 @@ test_expand_package_name() {
 
     # label-package
     assert_expansion_function "expand_package_name" "" "label-package" \
-    "//video/streamer2/stuff/\n//video/streamer2/stuff " \
+    "//video/streamer2/stuff \n//video/streamer2/stuff/" \
     "//video/streamer2/stu"
     assert_expansion_function "expand_package_name" "" "label-package" \
     "//video/notapackage/" \
@@ -357,7 +357,7 @@ test_complete_pattern() {
       "//video/streamer2/stu"
 
   assert_expansion_function "complete_pattern" "" label-package \
-      "//video/streamer2/stuff/\n//video/streamer2/stuff " \
+      "//video/streamer2/stuff \n//video/streamer2/stuff/" \
       "//video/streamer2/stu"
 
   assert_expansion_function "complete_pattern" "" command \
@@ -392,6 +392,19 @@ test_complete_pattern() {
       '' 'video/streamer2:ta'
   assert_expansion_function "complete_pattern" "video/" label \
       'with_special+_,=-.@~chars ' 'streamer2:with_s'
+
+  # Path expansion
+  if [[ -z $PACKAGE_PATH_PREFIX ]]; then
+      assert_expansion_function "complete_pattern" "" path \
+          "video/streamer2/BUILD \nvideo/streamer2/names/\nvideo/streamer2/stuff/\nvideo/streamer2/testing/" \
+          "video/streamer2/"
+  else
+      # When $PACKAGE_PATH_PREFIX is set, the "stuff" directory will not be in
+      # the same directory as the others, so we have to omit it.
+      assert_expansion_function "complete_pattern" "" path \
+          "video/streamer2/BUILD \nvideo/streamer2/names/\nvideo/streamer2/testing/" \
+          "video/streamer2/"
+  fi
 }
 
 #### TESTS #############################################################
@@ -707,6 +720,22 @@ test_target_expansion_in_package() {
                      'build :s'
 }
 
+test_filename_expansion_after_double_dash() {
+    make_packages
+    assert_expansion 'run :target -- vid' \
+                     'run :target -- video/'
+    assert_expansion 'run :target -- video/st' \
+                     'run :target -- video/streamer2/'
+    assert_expansion 'run :target -- video/streamer2/B' \
+                     'run :target -- video/streamer2/BUILD '
+    assert_expansion 'run :target -- video/streamer2/n' \
+                     'run :target -- video/streamer2/names/'
+
+    # Autocomplete arguments as well.
+    assert_expansion 'run :target -- --arg=video/streamer2/n' \
+                     'run :target -- --arg=video/streamer2/names/'
+}
+
 test_help() {
     # "Test that bazel help expands subcommand names"
     assert_expansion 'help qu' \
