@@ -13,6 +13,8 @@
 // limitations under the License.
 package com.google.devtools.common.options;
 
+import static com.google.devtools.common.options.OptionsParser.STARLARK_SKIPPED_PREFIXES;
+
 import com.google.common.base.Ascii;
 import com.google.common.base.Splitter;
 import com.google.common.base.Strings;
@@ -31,6 +33,12 @@ import java.util.regex.PatternSyntaxException;
 
 /** Some convenient converters used by blaze. Note: These are specific to blaze. */
 public final class Converters {
+  /**
+   * The name of the flag used for shorthand aliasing in blaze. {@see
+   * com.google.devtools.build.lib.analysis.config.CoreOptions#commandLineFlagAliases} for the
+   * option definition.
+   */
+  public static final String BLAZE_ALIASING_FLAG = "flag_alias";
 
   private static final ImmutableList<String> ENABLED_REPS =
       ImmutableList.of("true", "1", "yes", "t", "y");
@@ -457,6 +465,46 @@ public final class Converters {
     }
   }
 
+  /**
+   * A converter for command line flag aliases. It does additional validation on the name and value
+   * of the assignment to ensure they conform to the naming limitations.
+   */
+  public static class FlagAliasConverter extends AssignmentConverter {
+
+    @Override
+    public Map.Entry<String, String> convert(String input) throws OptionsParsingException {
+      Map.Entry<String, String> entry = super.convert(input);
+      String shortForm = entry.getKey();
+      String longForm = entry.getValue();
+
+      String cmdLineAlias = "--" + BLAZE_ALIASING_FLAG + "=" + input;
+
+      if (!Pattern.matches("([\\w])*", shortForm)) {
+        throw new OptionsParsingException(
+            shortForm + " should only consist of word characters to be a valid alias name.",
+            cmdLineAlias);
+      }
+      if (longForm.contains("=")) {
+        throw new OptionsParsingException(
+            "--" + BLAZE_ALIASING_FLAG + " does not support flag value assignment.", cmdLineAlias);
+      }
+
+      // Remove this check if native options are permitted to be aliased
+      longForm = "--" + longForm;
+      if (STARLARK_SKIPPED_PREFIXES.stream().noneMatch(longForm::startsWith)) {
+        throw new OptionsParsingException(
+            "--" + BLAZE_ALIASING_FLAG + " only supports Starlark build settings.", cmdLineAlias);
+      }
+
+      return entry;
+    }
+
+    @Override
+    public String getTypeDescription() {
+      return "a 'name=value' flag alias";
+    }
+  }
+
   /**
    * Base converter for assignments from a value to a list of values. Both the key type as well as
    * the type for all instances in the list of values are processed via passed converters.
