@@ -37,6 +37,7 @@ import java.util.Map;
 import java.util.function.Consumer;
 import java.util.function.Function;
 import java.util.function.Predicate;
+import javax.annotation.Nullable;
 
 /**
  * A parser for options. Typical use case in a main method:
@@ -92,15 +93,18 @@ public class OptionsParser implements OptionsParsingResult {
   }
 
   /**
-   * A cache for the parsed options data. Both keys and values are immutable, so
-   * this is always safe. Only access this field through the {@link
-   * #getOptionsData} method for thread-safety! The cache is very unlikely to
-   * grow to a significant amount of memory, because there's only a fixed set of
-   * options classes on the classpath.
+   * A cache for the parsed options data. Both keys and values are immutable, so this is always
+   * safe. Only access this field through the {@link #getOptionsData} method for thread-safety! The
+   * cache is very unlikely to grow to a significant amount of memory, because there's only a fixed
+   * set of options classes on the classpath.
    */
   private static final Map<ImmutableList<Class<? extends OptionsBase>>, OptionsData> optionsData =
       new HashMap<>();
 
+  /** Skipped prefixes for starlark options. */
+  public static final ImmutableList<String> STARLARK_SKIPPED_PREFIXES =
+      ImmutableList.of("--//", "--no//", "--@", "--no@");
+
   /**
    * Returns {@link OpaqueOptionsData} suitable for passing along to {@link
    * Builder#optionsData(OpaqueOptionsData optionsData)}.
@@ -114,9 +118,7 @@ public class OptionsParser implements OptionsParsingResult {
     return getOptionsDataInternal(optionsClasses);
   }
 
-  /**
-   * Returns the {@link OptionsData} associated with the given list of options classes.
-   */
+  /** Returns the {@link OptionsData} associated with the given list of options classes. */
   static synchronized OptionsData getOptionsDataInternal(
       List<Class<? extends OptionsBase>> optionsClasses) throws ConstructionException {
     ImmutableList<Class<? extends OptionsBase>> immutableOptionsClasses =
@@ -134,9 +136,7 @@ public class OptionsParser implements OptionsParsingResult {
     return result;
   }
 
-  /**
-   * Returns the {@link OptionsData} associated with the given options class.
-   */
+  /** Returns the {@link OptionsData} associated with the given options class. */
   static OptionsData getOptionsDataInternal(Class<? extends OptionsBase> optionsClass)
       throws ConstructionException {
     return getOptionsDataInternal(ImmutableList.of(optionsClass));
@@ -191,11 +191,10 @@ public class OptionsParser implements OptionsParsingResult {
 
     /** Skip all the prefixes associated with Starlark options */
     public Builder skipStarlarkOptionPrefixes() {
-      this.implBuilder
-          .skippedPrefix("--//")
-          .skippedPrefix("--no//")
-          .skippedPrefix("--@")
-          .skippedPrefix("--no@");
+      for (String prefix : STARLARK_SKIPPED_PREFIXES) {
+        this.implBuilder.skippedPrefix(prefix);
+      }
+
       return this;
     }
 
@@ -215,6 +214,12 @@ public class OptionsParser implements OptionsParsingResult {
       return this;
     }
 
+    /** Sets the string the parser should look for as an identifier for flag aliases. */
+    public Builder withAliasFlag(@Nullable String aliasFlag) {
+      this.implBuilder.withAliasFlag(aliasFlag);
+      return this;
+    }
+
     /** Returns a new {@link OptionsParser}. */
     public OptionsParser build() {
       return new OptionsParser(implBuilder.build(), allowResidue);
@@ -315,11 +320,14 @@ public class OptionsParser implements OptionsParsingResult {
   }
 
   /**
-   * The verbosity with which option help messages are displayed: short (just
-   * the name), medium (name, type, default, abbreviation), and long (full
-   * description).
+   * The verbosity with which option help messages are displayed: short (just the name), medium
+   * (name, type, default, abbreviation), and long (full description).
    */
-  public enum HelpVerbosity { LONG, MEDIUM, SHORT }
+  public enum HelpVerbosity {
+    LONG,
+    MEDIUM,
+    SHORT
+  }
 
   /**
    * Returns a description of all the options this parser can digest. In addition to {@link Option}
@@ -543,7 +551,8 @@ public class OptionsParser implements OptionsParsingResult {
     Preconditions.checkNotNull(visitor, "Missing visitor.");
 
     OptionsData data = impl.getOptionsData();
-    data.getOptionsClasses()
+    data
+        .getOptionsClasses()
         // List all options
         .stream()
         .flatMap(optionsClass -> OptionsData.getAllOptionDefinitionsForClass(optionsClass).stream())
@@ -817,4 +826,3 @@ public class OptionsParser implements OptionsParsingResult {
     return data.getUsesOnlyCoreTypes(optionsClass);
   }
 }
-
