@@ -20,6 +20,7 @@
 
 #include <WinIoCtl.h>
 #include <stdint.h>  // uint8_t
+#include <versionhelpers.h>
 #include <windows.h>
 
 #include <memory>
@@ -29,12 +30,31 @@
 
 #include "src/main/native/windows/util.h"
 
+#ifndef IO_REPARSE_TAG_PROJFS
+#define IO_REPARSE_TAG_PROJFS 0x9000001C
+#endif
+
 namespace bazel {
 namespace windows {
 
 using std::unique_ptr;
 using std::wstring;
 
+DWORD DetermineSymlinkPrivilegeFlag() {
+  DWORD val = 0;
+  DWORD valSize = sizeof(val);
+  // Check if developer mode is disabled
+  if (RegGetValueW(
+          HKEY_LOCAL_MACHINE,
+          L"SOFTWARE\\Microsoft\\Windows\\CurrentVersion\\AppModelUnlock",
+          L"AllowDevelopmentWithoutDevLicense", RRF_RT_DWORD, nullptr, &val,
+          &valSize) != ERROR_SUCCESS ||
+      val == 0) {
+    return 0;
+  }
+  return SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE;
+}
+
 wstring AddUncPrefixMaybe(const wstring& path) {
   return path.empty() || IsDevNull(path.c_str()) || HasUncPrefix(path.c_str())
              ? path
@@ -106,7 +126,7 @@ wstring GetLongPath(const WCHAR* path, unique_ptr<WCHAR[]>* result) {
   }
 
   std::wstring wpath(AddUncPrefixMaybe(path));
-  DWORD size = ::GetLongPathNameW(wpath.c_str(), NULL, 0);
+  DWORD size = ::GetLongPathNameW(wpath.c_str(), nullptr, 0);
   if (size == 0) {
     DWORD err_code = GetLastError();
     return MakeErrorMessage(WSTR(__FILE__), __LINE__, L"GetLongPathNameW", path,
@@ -210,14 +230,14 @@ int CreateJunction(const wstring& junction_name, const wstring& junction_target,
   // directory, or the path was invalid to begin with. Either way set `create`
   // to false, meaning we'll just attempt to open the path for metadata-reading
   // and check if it's a junction pointing to the desired target.
-  bool create = CreateDirectoryW(name.c_str(), NULL) != 0;
+  bool create = CreateDirectoryW(name.c_str(), nullptr) != 0;
 
   AutoHandle handle;
   if (create) {
     handle = CreateFileW(
-        name.c_str(), GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, NULL,
+        name.c_str(), GENERIC_READ | GENERIC_WRITE, FILE_SHARE_READ, nullptr,
         OPEN_EXISTING,
-        FILE_FLAG_OPEN_REPARSE_POINT | FILE_FLAG_BACKUP_SEMANTICS, NULL);
+        FILE_FLAG_OPEN_REPARSE_POINT | FILE_FLAG_BACKUP_SEMANTICS, nullptr);
   }
 
   if (!handle.IsValid()) {
@@ -230,8 +250,8 @@ int CreateJunction(const wstring& junction_name, const wstring& junction_target,
     create = false;
     handle = CreateFileW(
         name.c_str(), 0, FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
-        NULL, OPEN_EXISTING,
-        FILE_FLAG_OPEN_REPARSE_POINT | FILE_FLAG_BACKUP_SEMANTICS, NULL);
+        nullptr, OPEN_EXISTING,
+        FILE_FLAG_OPEN_REPARSE_POINT | FILE_FLAG_BACKUP_SEMANTICS, nullptr);
     if (!handle.IsValid()) {
       // We can't open the directory at all: either it disappeared, or it turned
       // into a file, or the path is invalid, or another process holds it open
@@ -371,7 +391,7 @@ int CreateJunction(const wstring& junction_name, const wstring& junction_target,
             handle, FSCTL_SET_REPARSE_POINT, reparse_buffer,
             reparse_buffer->ReparseDataLength +
                 offsetof(REPARSE_DATA_BUFFER, GenericReparseBuffer.DataBuffer),
-            NULL, 0, &bytes_returned, NULL)) {
+            nullptr, 0, &bytes_returned, nullptr)) {
       DWORD err = GetLastError();
       if (err == ERROR_DIR_NOT_EMPTY) {
         return CreateJunctionResult::kAlreadyExistsButNotJunction;
@@ -387,9 +407,9 @@ int CreateJunction(const wstring& junction_name, const wstring& junction_target,
     // The junction already exists. Check if it points to the right target.
 
     DWORD bytes_returned;
-    if (!::DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, NULL, 0,
+    if (!::DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, nullptr, 0,
                            reparse_buffer, MAXIMUM_REPARSE_DATA_BUFFER_SIZE,
-                           &bytes_returned, NULL)) {
+                           &bytes_returned, nullptr)) {
       DWORD err = GetLastError();
       // Some unknown error occurred.
       if (error) {
@@ -442,13 +462,21 @@ int CreateSymlink(const wstring& symlink_name, const wstring& symlink_target,
   }
 
   if (!CreateSymbolicLinkW(name.c_str(), target.c_str(),
-                           SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE)) {
-     // The flag SYMBOLIC_LINK_FLAG_ALLOW_UNPRIVILEGED_CREATE requires
-     // developer mode enabled, which we expect if using symbolic linking.
-     *error = MakeErrorMessage(
-               WSTR(__FILE__), __LINE__, L"CreateSymlink", symlink_target,
-               L"createSymbolicLinkW failed");
-     return CreateSymlinkResult::kError;
+                           symlinkPrivilegeFlag)) {
+    if (GetLastError() == ERROR_INVALID_PARAMETER) {
+      // We are on a version of Windows that does not support this flag.
+      // Retry without the flag and return to error handling if necessary.
+      if (CreateSymbolicLinkW(name.c_str(), target.c_str(), 0)) {
+        return CreateSymlinkResult::kSuccess;
+      }
+    }
+    *error = MakeErrorMessage(
+        WSTR(__FILE__), __LINE__, L"CreateSymlink", symlink_target,
+        GetLastError() == ERROR_PRIVILEGE_NOT_HELD
+            ? L"createSymbolicLinkW failed (permission denied). Either "
+              "Windows developer mode or admin privileges are required."
+            : L"createSymbolicLinkW failed");
+    return CreateSymlinkResult::kError;
   }
   return CreateSymlinkResult::kSuccess;
 }
@@ -466,9 +494,9 @@ int ReadSymlinkOrJunction(const wstring& path, wstring* result,
 
   AutoHandle handle(CreateFileW(
       AddUncPrefixMaybe(path).c_str(), 0,
-      FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, NULL,
+      FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE, nullptr,
       OPEN_EXISTING, FILE_FLAG_OPEN_REPARSE_POINT | FILE_FLAG_BACKUP_SEMANTICS,
-      NULL));
+      nullptr));
   if (!handle.IsValid()) {
     DWORD err = GetLastError();
     if (err == ERROR_SHARING_VIOLATION) {
@@ -491,9 +519,9 @@ int ReadSymlinkOrJunction(const wstring& path, wstring* result,
   uint8_t raw_buf[MAXIMUM_REPARSE_DATA_BUFFER_SIZE];
   PREPARSE_DATA_BUFFER buf = reinterpret_cast<PREPARSE_DATA_BUFFER>(raw_buf);
   DWORD bytes_returned;
-  if (!::DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, NULL, 0, buf,
+  if (!::DeviceIoControl(handle, FSCTL_GET_REPARSE_POINT, nullptr, 0, buf,
                          MAXIMUM_REPARSE_DATA_BUFFER_SIZE, &bytes_returned,
-                         NULL)) {
+                         nullptr)) {
     DWORD err = GetLastError();
     if (err == ERROR_NOT_A_REPARSE_POINT) {
       return ReadSymlinkOrJunctionResult::kNotALink;
@@ -524,6 +552,10 @@ int ReadSymlinkOrJunction(const wstring& path, wstring* result,
           p, buf->MountPointReparseBuffer.SubstituteNameLength / sizeof(WCHAR));
       return ReadSymlinkOrJunctionResult::kSuccess;
     }
+    case IO_REPARSE_TAG_PROJFS: {
+      // Virtual File System for Git
+      return ReadSymlinkOrJunctionResult::kNotALink;
+    }
     default:
       return ReadSymlinkOrJunctionResult::kUnknownLinkType;
   }
@@ -629,6 +661,15 @@ int DeletePath(const wstring& path, wstring* error) {
                                   GetLastError(), error);
   }
 
+  if (attr & FILE_ATTRIBUTE_READONLY) {
+    // Remove the read-only attribute.
+    attr &= ~FILE_ATTRIBUTE_READONLY;
+    if (!SetFileAttributesW(wpath, attr)) {
+      return GetResultFromErrorCode(L"SetFileAttributesW", path, GetLastError(),
+                                    error);
+    }
+  }
+
   if (attr & FILE_ATTRIBUTE_DIRECTORY) {
     // It's a directory or a junction, RemoveDirectoryW should be used.
     //
@@ -700,14 +741,6 @@ int DeletePath(const wstring& path, wstring* error) {
     }
   } else {
     // It's a regular file or symlink, DeleteFileW should be used.
-    if (attr & FILE_ATTRIBUTE_READONLY) {
-      // Remove the read-only attribute.
-      attr &= ~FILE_ATTRIBUTE_READONLY;
-      if (!SetFileAttributesW(wpath, attr)) {
-        return GetResultFromErrorCode(L"SetFileAttributesW", path,
-                                      GetLastError(), error);
-      }
-    }
     if (!DeleteFileW(wpath)) {
       // Failed to delete the file or symlink.
       return GetResultFromErrorCode(L"DeleteFileW", path,
