@@ -14,9 +14,9 @@
 
 package net.starlark.java.annot.processor.testsources;
 
-import com.google.devtools.build.lib.syntax.StarlarkValue;
 import net.starlark.java.annot.Param;
 import net.starlark.java.annot.StarlarkMethod;
+import net.starlark.java.eval.StarlarkValue;
 
 /**
  * Test case for a StarlarkMethod method which has no arguments when the annotation indicates it
@@ -28,7 +28,7 @@ public class ArgumentMissing implements StarlarkValue {
       name = "method_with_params",
       documented = false,
       parameters = {
-        @Param(name = "a_parameter", type = String.class, named = true),
+        @Param(name = "a_parameter", named = true),
       })
   public String methodWithParams() {
     return "bunny";
