@@ -14,26 +14,27 @@
 
 package net.starlark.java.annot.processor.testsources;
 
-import com.google.devtools.build.lib.syntax.StarlarkSemantics.FlagIdentifier;
-import com.google.devtools.build.lib.syntax.StarlarkValue;
 import net.starlark.java.annot.Param;
 import net.starlark.java.annot.StarlarkMethod;
+import net.starlark.java.eval.StarlarkValue;
 
 /**
  * Test case for a StarlarkMethod method which has both enablingFlag and disablingFlag specified.
  */
 public class EnablingAndDisablingFlag implements StarlarkValue {
 
+  static final String FOO = "-foo";
+
   @StarlarkMethod(
       name = "someMethod",
       documented = false,
       parameters = {
-        @Param(name = "one", type = String.class, named = true),
-        @Param(name = "two", type = Integer.class, named = true),
+        @Param(name = "one", named = true),
+        @Param(name = "two", named = true),
       },
-      enableOnlyWithFlag = FlagIdentifier.INCOMPATIBLE_APPLICABLE_LICENSES,
-      disableWithFlag = FlagIdentifier.INCOMPATIBLE_APPLICABLE_LICENSES)
-  public String someMethod(String one, Integer two) {
+      enableOnlyWithFlag = FOO,
+      disableWithFlag = FOO)
+  public String someMethod(String one, StarlarkInt two) {
     return "foo";
   }
 }
