@@ -14,9 +14,10 @@
 
 package net.starlark.java.annot.processor.testsources;
 
-import com.google.devtools.build.lib.syntax.StarlarkValue;
 import net.starlark.java.annot.Param;
 import net.starlark.java.annot.StarlarkMethod;
+import net.starlark.java.eval.StarlarkInt;
+import net.starlark.java.eval.StarlarkValue;
 
 /**
  * Test case for a StarlarkMethod method which has a positional parameter with no default value
@@ -31,7 +32,7 @@ public class NonDefaultParamAfterDefault implements StarlarkValue {
         @Param(name = "one", named = true, defaultValue = "1", positional = true),
         @Param(name = "two", named = true, positional = true)
       })
-  public Integer nonDefaultAfterDefault(Integer one, Integer two) {
+  public Integer nonDefaultAfterDefault(StarlarkInt one, StarlarkInt two) {
     return 42;
   }
-}
\ No newline at end of file
+}
