@@ -1445,9 +1445,10 @@ function test_java_info_constructor_e2e() {
   cat > java/com/google/foo/BUILD << EOF
 load(":my_rule.bzl", "my_rule")
 my_rule(
-  name = 'my_skylark_rule',
-  output_jar = 'my_skylark_rule_lib.jar',
-  source_jars = ['my_skylark_rule_src.jar'],
+  name = 'my_starlark_rule',
+  output_jar = 'my_starlark_rule_lib.jar',
+  output_source_jar = 'my_starlark_rule_lib-src.jar',
+  source_jars = ['my_starlark_rule_src.jar'],
 )
 EOF
 
@@ -1462,10 +1463,9 @@ def _impl(ctx):
   )
   source_jar = java_common.pack_sources(
     ctx.actions,
-    output_jar = ctx.file.output_jar,
+    output_source_jar = ctx.actions.declare_file(ctx.attr.output_source_jar),
     source_jars = ctx.files.source_jars,
     java_toolchain = ctx.attr._java_toolchain[java_common.JavaToolchainInfo],
-    host_javabase = ctx.attr._host_javabase[java_common.JavaRuntimeInfo],
   )
   javaInfo = JavaInfo(
     output_jar = ctx.file.output_jar,
@@ -1478,14 +1478,14 @@ my_rule = rule(
   implementation = _impl,
   attrs = {
     'output_jar' : attr.label(allow_single_file=True),
+    'output_source_jar' : attr.string(),
     'source_jars' : attr.label_list(allow_files=['.jar']),
     "_java_toolchain": attr.label(default = Label("@bazel_tools//tools/jdk:remote_toolchain")),
-    "_host_javabase": attr.label(default = Label("@bazel_tools//tools/jdk:current_host_java_runtime"))
   }
 )
 EOF
 
-  bazel build java/com/google/foo:my_skylark_rule >& "$TEST_log" || fail "Expected success"
+  bazel build java/com/google/foo:my_starlark_rule >& "$TEST_log" || fail "Expected success"
 }
 
 # This test builds a simple java deploy jar using remote singlejar and ijar
@@ -1626,4 +1626,105 @@ EOF
   expect_log "hello 123"
 }
 
+# Regression test for https://github.com/bazelbuild/bazel/issues/12605
+function test_java15_plugins() {
+  mkdir -p java/main
+  cat >java/main/BUILD <<EOF
+java_library(
+    name = "Anno",
+    srcs = ["Anno.java"],
+)
+
+java_plugin(
+    name = "Proc",
+    srcs = ["Proc.java"],
+    deps = [":Anno"],
+    processor_class = "ex.Proc",
+    generates_api = True,
+)
+
+java_library(
+    name = "C1",
+    srcs = ["C1.java"],
+    deps = [":Anno"],
+    plugins = [":Proc"],
+)
+
+java_library(
+    name = "C2",
+    srcs = ["C2.java"],
+    deps = [":C1"],
+)
+EOF
+
+  cat >java/main/C1.java <<EOF
+package ex;
+
+public class C1 {
+    @Anno
+    @Deprecated
+    public void m() {}
+}
+EOF
+
+
+  cat >java/main/C2.java <<EOF
+package ex;
+
+public class C2 {
+    public void m() {
+        new C1().m();
+    }
+}
+
+EOF
+
+  cat >java/main/Anno.java <<EOF
+package ex;
+
+import java.lang.annotation.ElementType;
+import java.lang.annotation.Retention;
+import java.lang.annotation.RetentionPolicy;
+import java.lang.annotation.Target;
+
+@Retention(RetentionPolicy.RUNTIME)
+@Target({ElementType.METHOD})
+public @interface Anno {}
+EOF
+
+  cat >java/main/Proc.java <<EOF
+package ex;
+
+import java.util.Set;
+
+import javax.annotation.processing.AbstractProcessor;
+import javax.annotation.processing.RoundEnvironment;
+import javax.annotation.processing.SupportedAnnotationTypes;
+import javax.annotation.processing.SupportedSourceVersion;
+import javax.lang.model.SourceVersion;
+import javax.lang.model.element.Element;
+import javax.lang.model.element.TypeElement;
+import javax.lang.model.util.Elements;
+import javax.tools.Diagnostic.Kind;
+
+@SupportedSourceVersion(SourceVersion.RELEASE_8)
+@SupportedAnnotationTypes("ex.Anno")
+public class Proc extends AbstractProcessor {
+    @Override
+    public boolean process(Set<? extends TypeElement> annotations, RoundEnvironment roundEnv) {
+        Elements els = processingEnv.getElementUtils();
+        for (Element el : roundEnv.getElementsAnnotatedWith(Anno.class)) {
+            if (els.isDeprecated(el)) {
+                processingEnv.getMessager().printMessage(Kind.WARNING, "deprecated");
+            }
+        }
+        return true;
+    }
+}
+EOF
+
+  bazel build //java/main:C2 &>"${TEST_log}" || fail "Expected to build"
+}
+
+
 run_suite "Java integration tests"
