@@ -192,7 +192,7 @@ EOF
       --remote_executor=grpc://localhost:${worker_port} \
       //a:test >& $TEST_log \
       || fail "Failed to build //a:test with remote execution"
-  expect_log "2 processes: 2 remote"
+  expect_log "6 processes: 4 internal, 2 remote"
   diff bazel-bin/a/test ${TEST_TMPDIR}/test_expected \
       || fail "Remote execution generated different result"
 }
@@ -398,7 +398,7 @@ EOF
       || fail "Expected success"
 
   mv gen1.log $TEST_log
-  expect_log "1 process: 1 local"
+  expect_log "2 processes: 1 internal, 1 local"
 }
 
 function test_local_fallback_with_sandbox_strategy_lists() {
@@ -421,7 +421,7 @@ EOF
       || fail "Expected success"
 
   mv gen1.log $TEST_log
-  expect_log "1 process: 1 .*-sandbox"
+  expect_log "2 processes: 1 internal, 1 .*-sandbox"
 }
 
 function test_local_fallback_to_sandbox_by_default() {
@@ -443,7 +443,7 @@ EOF
       || fail "Expected success"
 
   mv gen1.log $TEST_log
-  expect_log "1 process: 1 .*-sandbox"
+  expect_log "2 processes: 1 internal, 1 .*-sandbox"
 }
 
 function test_local_fallback_works_with_sandboxed_strategy() {
@@ -467,6 +467,54 @@ EOF
       && fail "Expected failure" || true
 }
 
+function test_local_fallback_if_no_remote_executor() {
+  # Test that when manually set --spawn_strategy that includes remote, but remote_executor isn't set, we ignore
+  # the remote strategy rather than reporting an error. See https://github.com/bazelbuild/bazel/issues/13340.
+  mkdir -p gen1
+  cat > gen1/BUILD <<'EOF'
+genrule(
+name = "gen1",
+srcs = [],
+outs = ["out1"],
+cmd = "touch \"$@\"",
+)
+EOF
+
+  bazel build \
+      --spawn_strategy=remote,local \
+      --build_event_text_file=gen1.log \
+      //gen1 >& $TEST_log \
+      || fail "Expected success"
+
+  mv gen1.log $TEST_log
+  expect_log "2 processes: 1 internal, 1 local"
+}
+
+function test_local_fallback_if_remote_executor_unavailable() {
+  # Test that when --remote_local_fallback is set and remote_executor is unavailable when build starts, we fallback to
+  # local strategy. See https://github.com/bazelbuild/bazel/issues/13487.
+  mkdir -p gen1
+  cat > gen1/BUILD <<'EOF'
+genrule(
+name = "gen1",
+srcs = [],
+outs = ["out1"],
+cmd = "touch \"$@\"",
+)
+EOF
+
+  bazel build \
+      --spawn_strategy=remote,local \
+      --remote_executor=grpc://noexist.invalid \
+      --remote_local_fallback \
+      --build_event_text_file=gen1.log \
+      //gen1 >& $TEST_log \
+      || fail "Expected success"
+
+  mv gen1.log $TEST_log
+  expect_log "2 processes: 1 internal, 1 local"
+}
+
 function is_file_uploaded() {
   h=$(shasum -a256 < $1)
   if [ -e "$cas_path/${h:0:64}" ]; then return 0; else return 1; fi
@@ -854,13 +902,13 @@ function test_treeartifact_in_runfiles() {
 load(":output_directory.bzl", "gen_output_dir", "gen_output_dir_test")
 
 gen_output_dir(
-    name = "skylark_output_dir",
+    name = "starlark_output_dir",
     outdir = "dir",
 )
 
 gen_output_dir_test(
-    name = "skylark_output_dir_test",
-    dir = ":skylark_output_dir",
+    name = "starlark_output_dir_test",
+    dir = ":starlark_output_dir",
 )
 EOF
      cat > a/output_directory.bzl <<'EOF'
@@ -908,14 +956,14 @@ EOF
      # a test into the sandboxing module.
      bazel test \
            --spawn_strategy=sandboxed \
-           //a:skylark_output_dir_test \
-           || fail "Failed to run //a:skylark_output_dir_test with sandboxing"
+           //a:starlark_output_dir_test \
+           || fail "Failed to run //a:starlark_output_dir_test with sandboxing"
 
      bazel test \
            --spawn_strategy=remote \
            --remote_executor=grpc://localhost:${worker_port} \
-           //a:skylark_output_dir_test \
-           || fail "Failed to run //a:skylark_output_dir_test with remote execution"
+           //a:starlark_output_dir_test \
+           || fail "Failed to run //a:starlark_output_dir_test with remote execution"
 }
 
 function test_downloads_minimal() {
@@ -1034,7 +1082,7 @@ EOF
     --remote_download_minimal \
     //a:remote >& $TEST_log || fail "Failed to build //a:remote"
 
-  expect_log "1 process: 1 remote"
+  expect_log "2 processes: 1 internal, 1 remote"
 
   bazel build \
     --genrule_strategy=remote \
@@ -1044,7 +1092,7 @@ EOF
 
   # Changing --remote_download_outputs to "all" should invalidate SkyFrames in-memory
   # caching and make it re-run the action.
-  expect_log "1 process: 1 remote"
+  expect_log "2 processes: 1 remote cache hit, 1 internal"
 }
 
 function test_downloads_minimal_native_prefetch() {
@@ -1099,7 +1147,7 @@ EOF
 
   # The genrule //a:generate-template should run remotely and //a:substitute-buchgr
   # should be a native action running locally.
-  expect_log "1 process: 1 remote"
+  expect_log "3 processes: 2 internal, 1 remote"
 
   outtxt="bazel-bin/a/substitute-buchgr.txt"
   [[ $(< ${outtxt}) == "Hello buchgr!" ]] \
@@ -1151,7 +1199,7 @@ EOF
     --remote_download_toplevel \
     //a:foobar >& $TEST_log || fail "Failed to build //a:foobar"
 
-  expect_log "1 process: 1 remote cache hit"
+  expect_log "2 processes: 1 remote cache hit, 1 internal"
 
   [[ -f bazel-bin/a/foobar.txt ]] \
   || fail "Expected toplevel output bazel-bin/a/foobar.txt to be re-downloaded"
@@ -1207,6 +1255,125 @@ EOF
   || fail "Expected runfile bazel-bin/a/create_bar.sh to be downloaded"
 }
 
+# Test that --remote_download_toplevel fetches inputs to symlink actions. In
+# particular, cc_binary links against a symlinked imported .so file, and only
+# the symlink is in the runfiles.
+function test_downloads_toplevel_symlinks() {
+  if [[ "$PLATFORM" == "darwin" ]]; then
+    # TODO(b/37355380): This test is disabled due to RemoteWorker not supporting
+    # setting SDKROOT and DEVELOPER_DIR appropriately, as is required of
+    # action executors in order to select the appropriate Xcode toolchain.
+    return 0
+  fi
+
+  mkdir -p a
+
+  cat > a/bar.cc <<'EOF'
+int f() {
+  return 42;
+}
+EOF
+
+  cat > a/foo.cc <<'EOF'
+extern int f();
+int main() { return f() == 42 ? 0 : 1; }
+EOF
+
+  cat > a/BUILD <<'EOF'
+cc_binary(
+  name = "foo",
+  srcs = ["foo.cc"],
+  deps = [":libbar_lib"],
+)
+
+cc_import(
+  name = "libbar_lib",
+  shared_library = ":libbar.so",
+)
+
+cc_binary(
+  name = "libbar.so",
+  srcs = ["bar.cc"],
+  linkshared = 1,
+  linkstatic = 1,
+)
+EOF
+
+  bazel build \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --remote_download_toplevel \
+    //a:foo || fail "Failed to build //a:foobar"
+
+  ./bazel-bin/a/foo${EXE_EXT} || fail "bazel-bin/a/foo${EXE_EXT} failed to run"
+}
+
+function test_symlink_outputs_not_allowed_with_minimial() {
+  mkdir -p a
+  cat > a/input.txt <<'EOF'
+Input file
+EOF
+  cat > a/BUILD <<'EOF'
+genrule(
+  name = "foo",
+  srcs = ["input.txt"],
+  outs = ["output.txt", "output_symlink"],
+  cmd = "cp $< $(location :output.txt) && ln -s output.txt $(location output_symlink)",
+)
+EOF
+
+  bazel build \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --remote_download_minimal \
+    //a:foo >& $TEST_log && fail "Expected failure to build //a:foo"
+  expect_log "Symlinks in action outputs are not yet supported"
+}
+
+# Regression test that --remote_download_toplevel does not crash when the
+# top-level output is a tree artifact.
+function test_downloads_toplevel_tree_artifact() {
+  if [[ "$PLATFORM" == "darwin" ]]; then
+    # TODO(b/37355380): This test is disabled due to RemoteWorker not supporting
+    # setting SDKROOT and DEVELOPER_DIR appropriately, as is required of
+    # action executors in order to select the appropriate Xcode toolchain.
+    return 0
+  fi
+
+  mkdir -p a
+
+  # We need the top-level output to be a tree artifact generated by a template
+  # action. This is one way to do that: generate a tree artifact of C++ source
+  # files, and then compile them with a cc_library / cc_binary rule.
+  #
+  # The default top-level output of a cc_binary is the final binary, which is
+  # not what we want. Instead, we use --output_groups=compilation_outputs to
+  # fetch the .o files as the top-level outputs.
+
+  cat > a/gentree.bzl <<'EOF'
+def _gentree(ctx):
+    out = ctx.actions.declare_directory("dir.cc")
+    ctx.actions.run_shell(
+        outputs = [out],
+        command = "mkdir -p %s && echo 'int main(int c, char** v){return 1;}' > %s/foo.cc" %
+            (out.path, out.path),
+    )
+    return DefaultInfo(files = depset([out]))
+
+gentree = rule(implementation = _gentree)
+EOF
+
+  cat > a/BUILD <<'EOF'
+load(":gentree.bzl", "gentree")
+gentree(name = "tree")
+cc_binary(name = "main", srcs = [":tree"])
+EOF
+
+  bazel build \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --remote_download_toplevel \
+    --output_groups=compilation_outputs \
+    //a:main || fail "Failed to build //a:main"
+}
+
 function test_downloads_toplevel_src_runfiles() {
   # Test that using --remote_download_toplevel with a non-generated (source)
   # runfile dependency works.
@@ -1321,6 +1488,41 @@ EOF
   expect_log "uri:.*bytestream://localhost"
 }
 
+function test_bytestream_uri_prefix() {
+  # Test that when --remote_bytestream_uri_prefix is set, bytestream://
+  # URIs do not contain the hostname that's part of --remote_executor.
+  # They should use a fixed value instead.
+  mkdir -p a
+  cat > a/success.sh <<'EOF'
+#!/bin/sh
+exit 0
+EOF
+  chmod 755 a/success.sh
+  cat > a/BUILD <<'EOF'
+sh_test(
+  name = "success_test",
+  srcs = ["success.sh"],
+)
+
+genrule(
+  name = "foo",
+  srcs = [],
+  outs = ["foo.txt"],
+  cmd = "echo \"foo\" > \"$@\"",
+)
+EOF
+
+  bazel test \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --remote_download_minimal \
+    --remote_bytestream_uri_prefix=example.com/my-instance-name \
+    --build_event_text_file=$TEST_log \
+    //a:foo //a:success_test || fail "Failed to test //a:foo //a:success_test"
+
+  expect_not_log 'uri:.*file://'
+  expect_log "uri:.*bytestream://example.com/my-instance-name/blobs"
+}
+
 # This test is derivative of test_bep_output_groups in
 # build_event_stream_test.sh, which verifies that successful output groups'
 # artifacts appear in BEP when a top-level target fails to build.
@@ -1541,6 +1743,30 @@ EOF
   assert_contains "test_case succeeded" "$TESTXML"
 }
 
+# Regression test that Bazel does not crash if remote execution is disabled,
+# but --remote_download_toplevel is enabled.
+function test_download_toplevel_no_remote_execution() {
+  bazel build --remote_download_toplevel \
+      || fail "Failed to run bazel build --remote_download_toplevel"
+}
+
+function test_download_toplevel_can_delete_directory_outputs() {
+  cat > BUILD <<'EOF'
+genrule(
+    name = 'g',
+    outs = ['out'],
+    cmd = "touch $@",
+)
+EOF
+  bazel build
+  mkdir $(bazel info bazel-genfiles)/out
+  touch $(bazel info bazel-genfiles)/out/f
+  bazel build \
+        --remote_download_toplevel \
+        --remote_executor=grpc://localhost:${worker_port} \
+        //:g \
+        || fail "should have worked"
+}
 
 function test_tag_no_remote_cache() {
   mkdir -p a
@@ -1656,7 +1882,7 @@ EOF
     --remote_default_exec_properties="build=1234" \
     //test:test >& $TEST_log || fail "Failed to build //a:remote"
 
-  expect_log "1 process: 1 remote"
+  expect_log "2 processes: 1 internal, 1 remote"
 
   bazel build \
     --remote_executor=grpc://localhost:${worker_port} \
@@ -1665,7 +1891,7 @@ EOF
 
   # Changing --remote_default_platform_properties value should invalidate SkyFrames in-memory
   # caching and make it re-run the action.
-  expect_log "1 process: 1 remote"
+  expect_log "2 processes: 1 internal, 1 remote"
 
   bazel  build \
     --remote_executor=grpc://localhost:${worker_port} \
@@ -1674,7 +1900,7 @@ EOF
 
   # The same value of --remote_default_platform_properties should NOT invalidate SkyFrames in-memory cache
   #  and make the action should not be re-run.
-  expect_log "0 processes"
+  expect_log "1 process: 1 internal"
 
   bazel shutdown
 
@@ -1685,7 +1911,7 @@ EOF
 
   # The same value of --remote_default_platform_properties should NOT invalidate SkyFrames od-disk cache
   #  and the action should not be re-run.
-  expect_log "0 processes"
+  expect_log "1 process: 1 internal"
 
   bazel build\
     --remote_executor=grpc://localhost:${worker_port} \
@@ -2016,6 +2242,102 @@ EOF
     @local_foo//:all
 }
 
+function test_remote_cache_intermediate_outputs() {
+  # test that remote cache is hit when intermediate output is not executable
+  touch WORKSPACE
+  cat > BUILD <<'EOF'
+genrule(
+  name = "dep",
+  srcs = [],
+  outs = ["dep"],
+  cmd = "echo 'dep' > $@",
+)
+
+genrule(
+  name = "test",
+  srcs = [":dep"],
+  outs = ["out"],
+  cmd = "cat $(SRCS) > $@",
+)
+EOF
+
+  bazel build \
+    --remote_cache=grpc://localhost:${worker_port} \
+    //:test >& $TEST_log || fail "Failed to build //:test"
+
+  bazel clean
+
+  bazel build \
+    --remote_cache=grpc://localhost:${worker_port} \
+    //:test >& $TEST_log || fail "Failed to build //:test"
+
+  expect_log "2 remote cache hit"
+}
+
+function test_remote_cache_intermediate_outputs_toplevel() {
+  # test that remote cache is hit when intermediate output is not executable in remote download toplevel mode
+  touch WORKSPACE
+  cat > BUILD <<'EOF'
+genrule(
+  name = "dep",
+  srcs = [],
+  outs = ["dep"],
+  cmd = "echo 'dep' > $@",
+)
+
+genrule(
+  name = "test",
+  srcs = [":dep"],
+  outs = ["out"],
+  cmd = "cat $(SRCS) > $@",
+)
+EOF
+
+  bazel build \
+    --remote_cache=grpc://localhost:${worker_port} \
+    --remote_download_toplevel \
+    //:test >& $TEST_log || fail "Failed to build //:test"
+
+  bazel clean
+
+  bazel build \
+    --remote_cache=grpc://localhost:${worker_port} \
+    --remote_download_toplevel \
+    //:test >& $TEST_log || fail "Failed to build //:test"
+
+  expect_log "2 remote cache hit"
+}
+
+function test_exclusive_tag() {
+  # Test that the exclusive tag works with the remote cache.
+  mkdir -p a
+  cat > a/success.sh <<'EOF'
+#!/bin/sh
+exit 0
+EOF
+  chmod 755 a/success.sh
+  cat > a/BUILD <<'EOF'
+sh_test(
+  name = "success_test",
+  srcs = ["success.sh"],
+  tags = ["exclusive"],
+)
+EOF
+
+  bazel test \
+    --incompatible_exclusive_test_sandboxed \
+    --remote_cache=grpc://localhost:${worker_port} \
+    //a:success_test || fail "Failed to test //a:success_test"
+
+  bazel test \
+    --incompatible_exclusive_test_sandboxed \
+    --remote_cache=grpc://localhost:${worker_port} \
+    --nocache_test_results \
+    //a:success_test >& $TEST_log || fail "Failed to test //a:success_test"
+
+  expect_log "remote cache hit"
+}
+
 # TODO(alpha): Add a test that fails remote execution when remote worker
 # supports sandbox.
 
@@ -2104,4 +2426,281 @@ EOF
   expect_log "INFO: Build completed successfully"
 }
 
+# This test uses the flag experimental_split_coverage_postprocessing. Without
+# the flag coverage won't work remotely. Without the flag, tests and coverage
+# post-processing happen in the same spawn, but only the runfiles tree of the
+# tests is made available to the spawn. The solution was not to merge the
+# runfiles tree which could cause its own problems but to split both into
+# different spawns. The reason why this only failed remotely and not locally was
+# because the coverage post-processing tool escaped the sandbox to find its own
+# runfiles. The error we would see here without the flag would be "Cannot find
+# runfiles". See #4685.
+function test_java_rbe_coverage_produces_report() {
+  mkdir -p java/factorial
+
+  JAVA_TOOLCHAIN="@bazel_tools//tools/jdk:toolchain"
+  add_to_bazelrc "build --java_toolchain=${JAVA_TOOLCHAIN}"
+  add_to_bazelrc "build --host_java_toolchain=${JAVA_TOOLCHAIN}"
+  if is_darwin; then
+      add_to_bazelrc "build --javabase=@openjdk14_darwin_archive//:runtime"
+      add_to_bazelrc "build --host_javabase=@openjdk14_darwin_archive//:runtime"
+  fi
+  JAVA_TOOLS_ZIP="released"
+  COVERAGE_GENERATOR_DIR="released"
+
+  cd java/factorial
+
+  cat > BUILD <<'EOF'
+java_library(
+    name = "fact",
+    srcs = ["Factorial.java"],
+)
+
+java_test(
+    name = "fact-test",
+    size = "small",
+    srcs = ["FactorialTest.java"],
+    test_class = "factorial.FactorialTest",
+    deps = [
+        ":fact",
+    ],
+)
+
+EOF
+
+  cat > Factorial.java <<'EOF'
+package factorial;
+
+public class Factorial {
+  public static int factorial(int x) {
+    return x <= 0 ? 1 : x * factorial(x-1);
+  }
+}
+EOF
+
+  cat > FactorialTest.java <<'EOF'
+package factorial;
+
+import static org.junit.Assert.*;
+
+import org.junit.Test;
+
+public class FactorialTest {
+  @Test
+  public void testFactorialOfZeroIsOne() throws Exception {
+    assertEquals(Factorial.factorial(3),6);
+  }
+}
+EOF
+  cd ../..
+
+  cat $(rlocation io_bazel/src/test/shell/bazel/testdata/jdk_http_archives) >> WORKSPACE
+
+  bazel coverage \
+    --test_output=all \
+    --experimental_fetch_all_coverage_outputs \
+    --experimental_split_coverage_postprocessing \
+    --spawn_strategy=remote \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --instrumentation_filter=//java/factorial \
+    //java/factorial:fact-test >& $TEST_log || fail "Shouldn't fail"
+
+  local expected_result="SF:java/factorial/Factorial.java
+FN:3,factorial/Factorial::<init> ()V
+FN:5,factorial/Factorial::factorial (I)I
+FNDA:0,factorial/Factorial::<init> ()V
+FNDA:1,factorial/Factorial::factorial (I)I
+FNF:2
+FNH:1
+BRDA:5,0,0,1
+BRDA:5,0,1,1
+BRF:2
+BRH:2
+DA:3,0
+DA:5,1
+LH:1
+LF:2
+end_of_record"
+
+  assert_equals "$expected_result" "$(cat bazel-testlogs/java/factorial/fact-test/coverage.dat)"
+}
+
+# Runs coverage with `cc_test` and RE then checks the coverage file is returned.
+# Older versions of gcov are not supported with bazel coverage and so will be skipped.
+# See the above `test_java_rbe_coverage_produces_report` for more information.
+function test_cc_rbe_coverage_produces_report() {
+  if [[ "$PLATFORM" == "darwin" ]]; then
+    # TODO(b/37355380): This test is disabled due to RemoteWorker not supporting
+    # setting SDKROOT and DEVELOPER_DIR appropriately, as is required of
+    # action executors in order to select the appropriate Xcode toolchain.
+    return 0
+  fi
+
+  # Check to see if intermediate files are supported, otherwise skip.
+  gcov --help | grep "\-i," || return 0
+
+  local test_dir="a/cc/coverage_test"
+  mkdir -p $test_dir
+
+  cat > "$test_dir"/BUILD <<'EOF'
+package(default_visibility = ["//visibility:public"])
+
+cc_library(
+    name = "hello-lib",
+    srcs = ["hello-lib.cc"],
+    hdrs = ["hello-lib.h"],
+)
+
+cc_binary(
+    name = "hello-world",
+    srcs = ["hello-world.cc"],
+    deps = [":hello-lib"],
+)
+
+cc_test(
+    name = "hello-test",
+    srcs = ["hello-world.cc"],
+    deps = [":hello-lib"],
+)
+
+EOF
+
+  cat > "$test_dir"/hello-lib.cc <<'EOF'
+#include "hello-lib.h"
+
+#include <iostream>
+
+using std::cout;
+using std::endl;
+using std::string;
+
+namespace hello {
+
+HelloLib::HelloLib(const string& greeting) : greeting_(new string(greeting)) {
+}
+
+void HelloLib::greet(const string& thing) {
+  cout << *greeting_ << " " << thing << endl;
+}
+
+}  // namespace hello
+
+EOF
+
+  cat > "$test_dir"/hello-lib.h <<'EOF'
+#ifndef HELLO_LIB_H_
+#define HELLO_LIB_H_
+
+#include <string>
+#include <memory>
+
+namespace hello {
+
+class HelloLib {
+ public:
+  explicit HelloLib(const std::string &greeting);
+
+  void greet(const std::string &thing);
+
+ private:
+  std::unique_ptr<const std::string> greeting_;
+};
+
+}  // namespace hello
+
+#endif  // HELLO_LIB_H_
+
+EOF
+
+  cat > "$test_dir"/hello-world.cc <<'EOF'
+#include "hello-lib.h"
+
+#include <string>
+
+using hello::HelloLib;
+using std::string;
+
+int main(int argc, char** argv) {
+  HelloLib lib("Hello");
+  string thing = "world";
+  if (argc > 1) {
+    thing = argv[1];
+  }
+  lib.greet(thing);
+  return 0;
+}
+
+EOF
+
+  bazel coverage \
+      --test_output=all \
+      --experimental_fetch_all_coverage_outputs \
+      --experimental_split_coverage_postprocessing \
+      --spawn_strategy=remote \
+      --remote_executor=grpc://localhost:${worker_port} \
+      //"$test_dir":hello-test >& $TEST_log \
+      || fail "Failed to run coverage for cc_test"
+
+  # Different gcov versions generate different outputs.
+  # Simply check if this is empty or not.
+  if [[ ! -s bazel-testlogs/a/cc/coverage_test/hello-test/coverage.dat ]]; then
+    echo "Coverage is empty. Failing now."
+    return 1
+  fi
+}
+
+# Test that when testing with --remote_download_minimal, Bazel doesn't
+# regenerate the test.xml if the action actually produced it. See
+# https://github.com/bazelbuild/bazel/issues/12554
+function test_remote_download_minimal_with_test_xml_generation() {
+  mkdir -p a
+
+  cat > a/BUILD <<'EOF'
+sh_test(
+    name = "test0",
+    srcs = ["test.sh"],
+)
+
+java_test(
+    name = "test1",
+    srcs = ["JavaTest.java"],
+    test_class = "JavaTest",
+)
+EOF
+
+  cat > a/test.sh <<'EOF'
+#!/bin/bash
+echo 'Hello'
+EOF
+  chmod a+x a/test.sh
+
+  cat > a/JavaTest.java <<'EOF'
+import org.junit.Test;
+
+public class JavaTest {
+    @Test
+    public void test() {}
+}
+EOF
+
+  bazel build \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --remote_download_minimal \
+    //a:test0 //a:test1 >& $TEST_log || fail "Failed to build"
+
+  bazel test \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --remote_download_minimal \
+    //a:test0 >& $TEST_log || fail "Failed to test"
+  # 2 remote spawns: 1 for executing the test, 1 for generating the test.xml
+  expect_log "2 processes: 2 remote"
+
+  bazel test \
+    --remote_executor=grpc://localhost:${worker_port} \
+    --remote_download_minimal \
+    //a:test1 >& $TEST_log || fail "Failed to test"
+  # only 1 remote spawn: test.xml is generated by junit
+  expect_log "2 processes: 1 internal, 1 remote"
+}
+
 run_suite "Remote execution and remote cache tests"
