@@ -54,6 +54,14 @@ public class LcovMergerTestUtils {
           "FNDA:0,file1-func3",
           "FNF:3",
           "FNH:2",
+          "BA:15,1",
+          "BA:15,2",
+          "BA:23,2",
+          "BA:23,1",
+          "BA:25,0",
+          "BA:25,0",
+          "BRF:6",
+          "BRH:2",
           "DA:10,3",
           "DA:11,3",
           "DA:12,30",
@@ -83,6 +91,14 @@ public class LcovMergerTestUtils {
           "FNDA:0,file1-func3",
           "FNF:3",
           "FNH:2",
+          "BA:15,1",
+          "BA:15,2",
+          "BA:23,2",
+          "BA:23,1",
+          "BA:25,0",
+          "BA:25,0",
+          "BRF:6",
+          "BRH:2",
           "DA:10,3",
           "DA:11,3",
           "DA:12,30",
@@ -113,6 +129,14 @@ public class LcovMergerTestUtils {
           "FNDA:2,file1-func3",
           "FNF:3",
           "FNH:3",
+          "BA:15,2",
+          "BA:15,1",
+          "BA:23,2",
+          "BA:23,2",
+          "BA:25,1",
+          "BA:25,2",
+          "BRF:6",
+          "BRH:4",
           "DA:10,2",
           "DA:11,2",
           "DA:12,20",
@@ -143,6 +167,14 @@ public class LcovMergerTestUtils {
           "FNDA:2,file1-func3",
           "FNF:3",
           "FNH:3",
+          "BA:15,2",
+          "BA:15,2",
+          "BA:23,2",
+          "BA:23,2",
+          "BA:25,1",
+          "BA:25,2",
+          "BRF:6",
+          "BRH:5",
           "DA:10,5",
           "DA:11,5",
           "DA:12,50",
@@ -185,6 +217,11 @@ public class LcovMergerTestUtils {
   static final int NR_LINES_HIT_TRACEFILE1 = 10;
   static final int NR_LINES_HIT_TRACEFILE2 = 13;
 
+  static final int NR_BRANCHES_FOUND = 6;
+  static final int NR_BRANCHES_HIT_TRACEFILE1 = 2;
+  static final int NR_BRANCHES_HIT_TRACEFILE2 = 4;
+  static final int NR_BRANCHES_HIT_MERGED = 5;
+
   static final int MAX_LINES_IN_FILE = 27;
 
   static int[] createLinesExecution1() {
@@ -256,28 +293,42 @@ public class LcovMergerTestUtils {
       }
     }
 
+    sourceFile.addBranch(15, BranchCoverage.create(15, 1));
+    sourceFile.addBranch(15, BranchCoverage.create(15, 2));
+    sourceFile.addBranch(23, BranchCoverage.create(23, 2));
+    sourceFile.addBranch(23, BranchCoverage.create(23, 1));
+    sourceFile.addBranch(25, BranchCoverage.create(25, 0));
+    sourceFile.addBranch(25, BranchCoverage.create(25, 0));
+
     return sourceFile;
   }
 
   // Create source file coverage data, excluding branch coverage
   static SourceFileCoverage createSourceFile2(int[] lineExecutionCount) {
-    SourceFileCoverage sourceFileCoverage = new SourceFileCoverage(SOURCE_FILENAME);
+    SourceFileCoverage sourceFile = new SourceFileCoverage(SOURCE_FILENAME);
 
-    sourceFileCoverage.addLineNumber(FUNC_1, FUNC_1_LINE_NR);
-    sourceFileCoverage.addFunctionExecution(FUNC_1, FUNC_1_NR_EXECUTED_LINES_TRACEFILE2);
+    sourceFile.addLineNumber(FUNC_1, FUNC_1_LINE_NR);
+    sourceFile.addFunctionExecution(FUNC_1, FUNC_1_NR_EXECUTED_LINES_TRACEFILE2);
 
-    sourceFileCoverage.addLineNumber(FUNC_2, FUNC_2_LINE_NR);
-    sourceFileCoverage.addFunctionExecution(FUNC_2, FUNC_2_NR_EXECECUTED_LINES_TRACEFILE2);
+    sourceFile.addLineNumber(FUNC_2, FUNC_2_LINE_NR);
+    sourceFile.addFunctionExecution(FUNC_2, FUNC_2_NR_EXECECUTED_LINES_TRACEFILE2);
 
-    sourceFileCoverage.addLineNumber(FUNC_3, FUNC_3_LINE_NR);
-    sourceFileCoverage.addFunctionExecution(FUNC_3, FUNC_3_NR_EXECUTED_LINES_TRACEFILE2);
+    sourceFile.addLineNumber(FUNC_3, FUNC_3_LINE_NR);
+    sourceFile.addFunctionExecution(FUNC_3, FUNC_3_NR_EXECUTED_LINES_TRACEFILE2);
 
     for (int line = FUNC_1_LINE_NR; line < MAX_LINES_IN_FILE; line++) {
       if (lineExecutionCount[line] >= 0) {
-        sourceFileCoverage.addLine(line, LineCoverage.create(line, lineExecutionCount[line], null));
+        sourceFile.addLine(line, LineCoverage.create(line, lineExecutionCount[line], null));
       }
     }
-    return sourceFileCoverage;
+
+    sourceFile.addBranch(15, BranchCoverage.create(15, 2));
+    sourceFile.addBranch(15, BranchCoverage.create(15, 1));
+    sourceFile.addBranch(23, BranchCoverage.create(23, 2));
+    sourceFile.addBranch(23, BranchCoverage.create(23, 2));
+    sourceFile.addBranch(25, BranchCoverage.create(25, 1));
+    sourceFile.addBranch(25, BranchCoverage.create(25, 2));
+    return sourceFile;
   }
 
   private static void assertLinesExecution_tracefile1(Map<Integer, LineCoverage> lines) {
@@ -325,6 +376,9 @@ public class LcovMergerTestUtils {
     assertThat(functionsExecution.get(FUNC_2)).isEqualTo(FUNC_2_NR_EXECUTED_LINES_TRACEFILE1);
     assertThat(functionsExecution.get(FUNC_3)).isEqualTo(FUNC_3_NR_EXECUTED_LINES_TRACEFILE1);
 
+    assertThat(sourceFile.nrBranchesFound()).isEqualTo(NR_BRANCHES_FOUND);
+    assertThat(sourceFile.nrBranchesHit()).isEqualTo(NR_BRANCHES_HIT_TRACEFILE1);
+
     assertLinesExecution_tracefile1(sourceFile.getLines());
 
     assertThat(sourceFile.nrOfInstrumentedLines()).isEqualTo(14);
@@ -346,6 +400,9 @@ public class LcovMergerTestUtils {
     assertThat(functionsExecution.get(FUNC_2)).isEqualTo(FUNC_2_NR_EXECECUTED_LINES_TRACEFILE2);
     assertThat(functionsExecution.get(FUNC_3)).isEqualTo(FUNC_3_NR_EXECUTED_LINES_TRACEFILE2);
 
+    assertThat(sourceFile.nrBranchesFound()).isEqualTo(NR_BRANCHES_FOUND);
+    assertThat(sourceFile.nrBranchesHit()).isEqualTo(NR_BRANCHES_HIT_TRACEFILE2);
+
     assertLines_tracefile2(sourceFile.getLines());
 
     assertThat(sourceFile.nrOfInstrumentedLines()).isEqualTo(14);
@@ -401,6 +458,9 @@ public class LcovMergerTestUtils {
     assertMergedFunctionsExecution(merged.getFunctionsExecution());
     assertMergedLines(merged.getLines(), linesExecution1, linesExecution2);
 
+    assertThat(merged.nrBranchesFound()).isEqualTo(NR_BRANCHES_FOUND);
+    assertThat(merged.nrBranchesHit()).isEqualTo(NR_BRANCHES_HIT_MERGED);
+
     assertThat(merged.nrFunctionsFound()).isEqualTo(NR_FUNCTIONS_FOUND);
     assertThat(merged.nrFunctionsHit()).isEqualTo(NR_FUNCTIONS_FOUND);
     assertThat(merged.nrOfLinesWithNonZeroExecution()).isEqualTo(14);
