@@ -189,10 +189,13 @@ struct Duration {
   bool FromString(const wchar_t* str);
 };
 
+enum class MainType { kTestWrapperMain, kXmlWriterMain };
+enum class DeleteAfterwards { kEnabled, kDisabled };
+
 void WriteStdout(const std::string& s) {
   DWORD written;
   WriteFile(GetStdHandle(STD_OUTPUT_HANDLE), s.c_str(), s.size(), &written,
-            NULL);
+            nullptr);
 }
 
 void LogError(const int line) {
@@ -319,9 +322,10 @@ std::wstring AsMixedPath(const std::wstring& path) {
 }
 
 bool IsReadableFile(const Path& p) {
-  HANDLE h = CreateFileW(AddUncPrefixMaybe(p).c_str(), GENERIC_READ,
-                         FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
-                         NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+  HANDLE h =
+      CreateFileW(AddUncPrefixMaybe(p).c_str(), GENERIC_READ,
+                  FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                  nullptr, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, nullptr);
   if (h == INVALID_HANDLE_VALUE) {
     return false;
   }
@@ -400,7 +404,7 @@ bool SetPathEnv(const wchar_t* name, const Path& path) {
 }
 
 bool UnsetEnv(const wchar_t* name) {
-  if (SetEnvironmentVariableW(name, NULL) != 0) {
+  if (SetEnvironmentVariableW(name, nullptr) != 0) {
     return true;
   } else {
     DWORD err = GetLastError();
@@ -409,6 +413,11 @@ bool UnsetEnv(const wchar_t* name) {
   }
 }
 
+bool AddCurrentDirectoryToPATH() {
+  std::wstring path;
+  return GetEnv(L"PATH", &path) && SetEnv(L"PATH", L".;" + path);
+}
+
 bool GetCwd(Path* result) {
   static constexpr size_t kSmallBuf = MAX_PATH;
   WCHAR value[kSmallBuf];
@@ -776,8 +785,8 @@ bool CreateZipBuilder(const Path& zip, const ZipEntryPaths& entry_paths,
 
 bool OpenFileForWriting(const Path& path, bazel::windows::AutoHandle* result) {
   HANDLE h = CreateFileW(AddUncPrefixMaybe(path).c_str(), GENERIC_WRITE,
-                         FILE_SHARE_READ | FILE_SHARE_DELETE, NULL,
-                         CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, NULL);
+                         FILE_SHARE_READ | FILE_SHARE_DELETE, nullptr,
+                         CREATE_ALWAYS, FILE_ATTRIBUTE_NORMAL, nullptr);
   if (h == INVALID_HANDLE_VALUE) {
     DWORD err = GetLastError();
     LogErrorWithArgAndValue(__LINE__, "Failed to open file", path.Get(), err);
@@ -789,9 +798,10 @@ bool OpenFileForWriting(const Path& path, bazel::windows::AutoHandle* result) {
 
 bool OpenExistingFileForRead(const Path& abs_path,
                              bazel::windows::AutoHandle* result) {
-  HANDLE h = CreateFileW(AddUncPrefixMaybe(abs_path).c_str(), GENERIC_READ,
-                         FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
-                         NULL, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL);
+  HANDLE h =
+      CreateFileW(AddUncPrefixMaybe(abs_path).c_str(), GENERIC_READ,
+                  FILE_SHARE_READ | FILE_SHARE_WRITE | FILE_SHARE_DELETE,
+                  nullptr, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, nullptr);
   if (h == INVALID_HANDLE_VALUE) {
     DWORD err = GetLastError();
     LogErrorWithArgAndValue(__LINE__, "Failed to open file", abs_path.Get(),
@@ -816,7 +826,7 @@ bool ReadFromFile(HANDLE handle, uint8_t* dest, DWORD max_read) {
   DWORD read = 0;
   do {
     if (!ReadFile(handle, dest + total_read, max_read - total_read, &read,
-                  NULL)) {
+                  nullptr)) {
       DWORD err = GetLastError();
       LogErrorWithValue(__LINE__, "Failed to read file", err);
       return false;
@@ -832,7 +842,7 @@ bool WriteToFile(HANDLE output, const void* buffer, const size_t size) {
   while (total_written < size) {
     DWORD written;
     if (!WriteFile(output, static_cast<const uint8_t*>(buffer) + total_written,
-                   size - total_written, &written, NULL)) {
+                   size - total_written, &written, nullptr)) {
       DWORD err = GetLastError();
       LogErrorWithValue(__LINE__, "Failed to write file", err);
       return false;
@@ -855,7 +865,7 @@ bool AppendFileTo(const Path& file, const size_t total_size, HANDLE output) {
   while (true) {
     // Read at most `buf_size` many bytes from the input file.
     DWORD read = 0;
-    if (!ReadFile(input, buffer.get(), buf_size, &read, NULL)) {
+    if (!ReadFile(input, buffer.get(), buf_size, &read, nullptr)) {
       DWORD err = GetLastError();
       LogErrorWithArgAndValue(__LINE__, "Failed to read file", file.Get(), err);
       return false;
@@ -885,7 +895,7 @@ std::string GetMimeType(const std::string& filename) {
   char data[1000];
   DWORD data_size = 1000 * sizeof(char);
   if (RegGetValueA(HKEY_CLASSES_ROOT, filename.c_str() + pos, "Content Type",
-                   RRF_RT_REG_SZ, NULL, data, &data_size) == ERROR_SUCCESS) {
+                   RRF_RT_REG_SZ, nullptr, data, &data_size) == ERROR_SUCCESS) {
     return data;
   }
   // The file extension is unknown, or it does not have a "Content Type" value,
@@ -1211,7 +1221,7 @@ bool StartSubprocess(const Path& path, const std::wstring& args,
                      LARGE_INTEGER* start_time,
                      bazel::windows::WaitableProcess* process) {
   SECURITY_ATTRIBUTES inheritable_handle_sa = {sizeof(SECURITY_ATTRIBUTES),
-                                               NULL, TRUE};
+                                               nullptr, TRUE};
 
   // Create a pipe to stream the output of the subprocess to this process.
   // The subprocess inherits two copies of the writing end (one for stdout, one
@@ -1243,7 +1253,7 @@ bool StartSubprocess(const Path& path, const std::wstring& args,
   bazel::windows::AutoHandle devnull_read(CreateFileW(
       L"NUL", GENERIC_READ,
       FILE_SHARE_WRITE | FILE_SHARE_READ | FILE_SHARE_DELETE,
-      &inheritable_handle_sa, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, NULL));
+      &inheritable_handle_sa, OPEN_EXISTING, FILE_ATTRIBUTE_NORMAL, nullptr));
   if (devnull_read == INVALID_HANDLE_VALUE) {
     DWORD err = GetLastError();
     LogErrorWithValue(__LINE__, "CreateFileW", err);
@@ -1405,7 +1415,7 @@ bool TeeImpl::Create(bazel::windows::AutoHandle* input,
                      std::unique_ptr<Tee>* result) {
   std::unique_ptr<TeeImpl> tee(new TeeImpl(input, output1, output2));
   bazel::windows::AutoHandle thread(
-      CreateThread(NULL, 0, ThreadFunc, tee.get(), 0, NULL));
+      CreateThread(nullptr, 0, ThreadFunc, tee.get(), 0, nullptr));
   if (!thread.IsValid()) {
     return false;
   }
@@ -1421,10 +1431,10 @@ bool TeeImpl::MainFunc() const {
   static constexpr size_t kBufferSize = 0x10000;
   DWORD read;
   uint8_t content[kBufferSize];
-  while (ReadFile(input_, content, kBufferSize, &read, NULL)) {
+  while (ReadFile(input_, content, kBufferSize, &read, nullptr)) {
     DWORD written;
-    if (read > 0 && (!WriteFile(output1_, content, read, &written, NULL) ||
-                     !WriteFile(output2_, content, read, &written, NULL))) {
+    if (read > 0 && (!WriteFile(output1_, content, read, &written, nullptr) ||
+                     !WriteFile(output2_, content, read, &written, nullptr))) {
       return false;
     }
   }
@@ -1598,9 +1608,16 @@ std::string CreateErrorTag(int exit_code) {
   }
 }
 
-bool ShouldCreateXml(const Path& xml_log, bool* result) {
+bool ShouldCreateXml(const Path& xml_log, const MainType main_type,
+                     bool* result) {
   *result = true;
 
+  // If running from the xml generator binary, we should always create the xml
+  // file.
+  if (main_type == MainType::kXmlWriterMain) {
+    return true;
+  }
+
   DWORD attr = GetFileAttributesW(AddUncPrefixMaybe(xml_log).c_str());
   if (attr != INVALID_FILE_ATTRIBUTES) {
     // The XML file already exists, maybe the test framework wrote it.
@@ -1625,9 +1642,10 @@ bool ShouldCreateXml(const Path& xml_log, bool* result) {
 
 bool CreateXmlLog(const Path& output, const Path& test_outerr,
                   const Duration duration, const int exit_code,
-                  const bool delete_afterwards) {
+                  const DeleteAfterwards delete_afterwards,
+                  const MainType main_type) {
   bool should_create_xml;
-  if (!ShouldCreateXml(output, &should_create_xml)) {
+  if (!ShouldCreateXml(output, main_type, &should_create_xml)) {
     LogErrorWithArg(__LINE__, "Failed to decide if XML log is needed",
                     output.Get());
     return false;
@@ -1640,7 +1658,7 @@ bool CreateXmlLog(const Path& output, const Path& test_outerr,
     // Delete the test's outerr file after we have the XML file.
     // We don't care if this succeeds or not, because the outerr file is not a
     // declared output.
-    if (delete_afterwards) {
+    if (delete_afterwards == DeleteAfterwards::kEnabled) {
       DeleteFileW(test_outerr.Get().c_str());
     }
   });
@@ -1753,7 +1771,7 @@ Path Path::Dirname() const {
 IFStream* IFStreamImpl::Create(HANDLE handle, DWORD page_size) {
   std::unique_ptr<uint8_t[]> data(new uint8_t[page_size * 2]);
   DWORD read;
-  if (!ReadFile(handle, data.get(), page_size * 2, &read, NULL)) {
+  if (!ReadFile(handle, data.get(), page_size * 2, &read, nullptr)) {
     DWORD err = GetLastError();
     if (err == ERROR_BROKEN_PIPE) {
       read = 0;
@@ -1779,7 +1797,7 @@ int IFStreamImpl::Get() {
   // Overwrite the *active* page: we are about to move off of it.
   DWORD offs = (pos_ < page_size_) ? 0 : page_size_;
   DWORD read;
-  if (!ReadFile(handle_, pages_.get() + offs, page_size_, &read, NULL)) {
+  if (!ReadFile(handle_, pages_.get() + offs, page_size_, &read, nullptr)) {
     DWORD err = GetLastError();
     if (err == ERROR_BROKEN_PIPE) {
       // The stream is reading from a pipe, and there's no more data.
@@ -1883,9 +1901,10 @@ int TestWrapperMain(int argc, wchar_t** argv) {
   Path test_path, exec_root, srcdir, tmpdir, test_outerr, xml_log;
   UndeclaredOutputs undecl;
   std::wstring args;
-  if (!ParseArgs(argc, argv, &argv0, &test_path_arg, &args) ||
-      !PrintTestLogStartMarker() || !GetCwd(&exec_root) ||
-      !ExportUserName() || !ExportSrcPath(exec_root, &srcdir) ||
+  if (!AddCurrentDirectoryToPATH() ||
+      !ParseArgs(argc, argv, &argv0, &test_path_arg, &args) ||
+      !PrintTestLogStartMarker() || !GetCwd(&exec_root) || !ExportUserName() ||
+      !ExportSrcPath(exec_root, &srcdir) ||
       !FindTestBinary(argv0, exec_root, test_path_arg, srcdir, &test_path) ||
       !ChdirToRunfiles(exec_root, srcdir) ||
       !ExportTmpPath(exec_root, &tmpdir) || !ExportHome(tmpdir) ||
@@ -1898,7 +1917,8 @@ int TestWrapperMain(int argc, wchar_t** argv) {
 
   Duration test_duration;
   int result = RunSubprocess(test_path, args, test_outerr, &test_duration);
-  if (!CreateXmlLog(xml_log, test_outerr, test_duration, result, true) ||
+  if (!CreateXmlLog(xml_log, test_outerr, test_duration, result,
+                    DeleteAfterwards::kEnabled, MainType::kTestWrapperMain) ||
       !ArchiveUndeclaredOutputs(undecl) ||
       !CreateUndeclaredOutputsAnnotations(undecl.annotations_dir,
                                           undecl.annotations)) {
@@ -1915,7 +1935,8 @@ int XmlWriterMain(int argc, wchar_t** argv) {
   if (!GetCwd(&cwd) ||
       !ParseXmlWriterArgs(argc, argv, cwd, &test_outerr, &test_xml_log,
                           &duration, &exit_code) ||
-      !CreateXmlLog(test_xml_log, test_outerr, duration, exit_code, false)) {
+      !CreateXmlLog(test_xml_log, test_outerr, duration, exit_code,
+                    DeleteAfterwards::kDisabled, MainType::kXmlWriterMain)) {
     return 1;
   }
 
